"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VisualVariableScalesBuilder = exports.CategorySummary = exports.AttributeCategories = exports.AttributeExtents = exports.Extent = void 0;
const multi_set_1 = __importDefault(require("mnemonist/multi-set"));
const palette_1 = __importDefault(require("iwanthue/palette"));
const d3_scale_1 = require("d3-scale");
/**
 * Constants.
 */
const CATEGORY_MAX_COUNT = 10;
/**
 * Helper functions.
 */
function isValidNumber(value) {
    return typeof value === 'number' && !Number.isNaN(value);
}
/**
 * Helper classes.
 */
class Extent {
    constructor() {
        this.min = Infinity;
        this.max = -Infinity;
    }
    add(value) {
        if (value < this.min)
            this.min = value;
        if (value > this.max)
            this.max = value;
    }
    isConstant() {
        return this.min === Infinity || this.min === this.max;
    }
}
exports.Extent = Extent;
class AttributeExtents {
    constructor(names) {
        this.attributes = {};
        // NOTE: this naturally deduplicates names
        names.forEach((name) => (this.attributes[name] = new Extent()));
    }
    add(attributes) {
        for (const name in this.attributes) {
            const value = attributes[name];
            if (!isValidNumber(value))
                continue;
            this.attributes[name].add(value);
        }
    }
}
exports.AttributeExtents = AttributeExtents;
class AttributeCategories {
    constructor(names) {
        this.attributes = {};
        // NOTE: this naturally deduplicates names
        names.forEach((name) => (this.attributes[name] = new multi_set_1.default()));
    }
    add(attributes) {
        for (const name in this.attributes) {
            this.attributes[name].add(attributes[name]);
        }
    }
}
exports.AttributeCategories = AttributeCategories;
class CategorySummary {
    constructor(name, palette, overflowing = false) {
        this.name = name;
        this.palette = palette;
        this.overflowing = overflowing;
    }
    static fromTopValues(name, frequencies, defaultColor, maxCount = CATEGORY_MAX_COUNT) {
        const count = Math.min(maxCount, frequencies.dimension);
        const topValues = frequencies.top(count);
        const overflowing = count < frequencies.dimension;
        const values = topValues.map((item) => item[0]);
        const palette = palette_1.default.generateFromValues(name, values, { defaultColor });
        return new CategorySummary(name, palette, overflowing);
    }
    static fromColorEntries(name, entries, defaultColor) {
        const palette = palette_1.default.fromEntries(name, entries, defaultColor);
        return new CategorySummary(name, palette);
    }
}
exports.CategorySummary = CategorySummary;
class VisualVariableScalesBuilder {
    constructor(visualVariables, maxCategoryColors = CATEGORY_MAX_COUNT) {
        this.variables = visualVariables;
        this.maxCategoryColors = maxCategoryColors;
        const nodeExtentAttributes = [];
        const nodeCategoryAttributes = [];
        const edgeExtentAttributes = [];
        const edgeCategoryAttributes = [];
        for (const variableName in visualVariables) {
            const variable = visualVariables[variableName];
            if (variableName.startsWith('node')) {
                if (variable.type === 'category') {
                    if (!variable.palette)
                        nodeCategoryAttributes.push(variable.attribute);
                }
                else if (variable.type === 'continuous') {
                    nodeExtentAttributes.push(variable.attribute);
                }
            }
            else if (variableName.startsWith('edge')) {
                if (variable.type === 'category') {
                    if (!variable.palette)
                        edgeCategoryAttributes.push(variable.attribute);
                }
                else if (variable.type === 'continuous') {
                    if (variable.range[0] !== variable.range[1])
                        edgeExtentAttributes.push(variable.attribute);
                }
            }
        }
        this.nodeExtents = new AttributeExtents(nodeExtentAttributes);
        this.edgeExtents = new AttributeExtents(edgeExtentAttributes);
        this.nodeCategories = new AttributeCategories(nodeCategoryAttributes);
        this.edgeCategories = new AttributeCategories(edgeCategoryAttributes);
    }
    readGraph(graph) {
        graph.forEachNode((node, attr) => {
            this.nodeExtents.add(attr);
            this.nodeCategories.add(attr);
        });
        graph.forEachEdge((edge, attr) => {
            this.edgeExtents.add(attr);
            this.edgeCategories.add(attr);
        });
    }
    build() {
        const scales = {};
        for (const variableName in this.variables) {
            const variable = this.variables[variableName];
            let scale = null;
            // Raw variables
            if (variable.type === 'raw') {
                scale = (attr) => attr[variable.attribute] || variable.default;
            }
            // Category variables
            else if (variable.type === 'category') {
                const categories = variableName.startsWith('node')
                    ? this.nodeCategories
                    : this.edgeCategories;
                const summary = variable.palette
                    ? CategorySummary.fromColorEntries(variable.attribute, variable.palette, variable.default || '#ccc')
                    : CategorySummary.fromTopValues(variable.attribute, categories.attributes[variable.attribute], variable.default || '#ccc', this.maxCategoryColors);
                const palette = summary.palette;
                scale = (attr) => palette.get(attr[variable.attribute]);
                scale.summary = summary;
            }
            // Continuous variables
            else if (variable.type === 'continuous') {
                const extents = variableName.startsWith('node')
                    ? this.nodeExtents
                    : this.edgeExtents;
                const extent = extents.attributes[variable.attribute];
                if (variable.range[0] === variable.range[1] || extent.isConstant()) {
                    scale = () => variable.range[0];
                }
                else {
                    const continuousScale = (0, d3_scale_1.scaleLinear)()
                        .domain([extent.min, extent.max])
                        .range(variable.range);
                    scale = (attr) => continuousScale(attr[variable.attribute]);
                }
            }
            if (scale)
                scales[variableName] = scale;
        }
        return scales;
    }
    inferLabelRenderedSizeThreshold() {
        const attribute = this.variables.nodeSize.attribute;
        const extent = this.nodeExtents.attributes[attribute];
        return Math.min(6, extent.max);
    }
}
exports.VisualVariableScalesBuilder = VisualVariableScalesBuilder;
//# sourceMappingURL=visual-variables.js.map